@@ -237,7 +237,6 @@ class ConfManager(LoggerMixin, threading.Thread):
                 vm.kill_check_ctxt_process()
             except Exception:
                 self.log_exception("Error killing ctxt processes in VM: %s" % vm.id)
-            vm.configured = None
 
     def run(self):
         self.log_info("Starting the ConfManager Thread")
@@ -256,6 +255,7 @@ class ConfManager(LoggerMixin, threading.Thread):
                 if self.ansible_process and self.ansible_process.is_alive():
                     self.log_info("Stopping pending Ansible process.")
                     self.ansible_process.terminate()
+                    self._stop_thread = True
 
                 # Set as unconfigured all non finished ctxt VMs
                 for vm in self.inf.get_vm_list():
@@ -564,8 +564,9 @@ class ConfManager(LoggerMixin, threading.Thread):
         out.write('IM_MASTER_HOSTNAME=' + master_name + '\n')
         out.write('IM_MASTER_FQDN=' + master_name + "." + masterdom + '\n')
         out.write('IM_MASTER_DOMAIN=' + masterdom + '\n')
-        out.write('IM_INFRASTRUCTURE_ID=' + self.inf.id + '\n\n')
-        out.write('IM_INFRASTRUCTURE_RADL=' + self.inf.get_json_radl() + '\n\n')
+        out.write('IM_INFRASTRUCTURE_ID=' + self.inf.id + '\n')
+        out.write('IM_INFRASTRUCTURE_RADL=' + self.inf.get_json_radl() + '\n')
+        out.write('IM_INFRASTRUCTURE_AUTH=' + self.inf.get_auth() + '\n\n')
 
         if windows:
             out.write('[windows]\n' + windows + "\n")
