@@ -20,7 +20,7 @@ import paramiko
 try:
     import scp
 except Exception:
-    print("WARN: Python Azure SDK not correctly installed. AzureCloudConnector will not work!.")
+    print("WARN: SCP library not correctly installed. Some sftp functions will not work!.")
 import os
 try:
     from StringIO import StringIO
@@ -52,6 +52,9 @@ class ThreadSSH(Thread):
         self.client = None
         self.proxy = None
 
+    def __del__(self):
+        self.close()
+
     def close(self):
         """
         Close the SSH client connection
@@ -121,6 +124,9 @@ class SSH:
             self.private_key_obj = paramiko.RSAKey.from_private_key(
                 private_key_obj)
 
+    def __del__(self):
+        self.close()
+
     def close(self):
         """
         Close the SSH client connection
