@@ -437,7 +437,7 @@ class EC2CloudConnector(CloudConnector):
         subnet_id = None
 
         for vpc in conn.get_all_vpcs():
-            if vpc.is_default or vpc.tags['Name'] == "default":
+            if vpc.is_default or ('Name' in vpc.tags and vpc.tags['Name'] == "default"):
                 vpc_id = vpc.id
                 for subnet in conn.get_all_subnets(filters={"vpcId": vpc_id}):
                     subnet_id = subnet.id
@@ -1525,9 +1525,13 @@ class EC2CloudConnector(CloudConnector):
         else:
             return (False, "The instance has not been found")
 
+        new_system = self.resize_vm_radl(vm, radl)
+        if not new_system:
+            return (True, "")
+
         success = True
-        if radl.systems:
-            instance_type = self.get_instance_type(radl.systems[0])
+        if new_system:
+            instance_type = self.get_instance_type(new_system)
 
             if instance_type and instance.instance_type != instance_type.name:
                 stopped = self.waitStop(instance)
