@@ -17,6 +17,7 @@
 import uuid
 import os
 import time
+import re
 
 try:
     from libcloud.compute.base import NodeSize, NodeState
@@ -545,7 +546,9 @@ class GCECloudConnector(LibCloudCloudConnector):
         if tags:
             args['ex_labels'] = {}
             for key, value in tags.items():
-                args['ex_labels'][key.replace("-", "_").lower()] = value
+                value = re.sub('[^a-zA-Z0-9\-_]', '', value)
+                key = re.sub('[^a-zA-Z0-9\-_]', '', key)
+                args['ex_labels'][key.lower()] = value.lower()
 
         # include the SSH_KEYS
         username = system.getValue('disk.0.os.credentials.username')
@@ -987,6 +990,6 @@ class GCECloudConnector(LibCloudCloudConnector):
         for location in driver.list_locations():
             if not region or region == location.name:
                 for image in gce_images:
-                    images.append({"uri": "gce://%s/%s" % (location.name, image.id),
+                    images.append({"uri": "gce://%s/%s" % (location.name, image.name),
                                    "name": "%s/%s" % (location.name, image.name)})
         return images
