@@ -18,7 +18,7 @@ import defusedxml.xmlrpc
 defusedxml.xmlrpc.monkey_patch()
 
 try:
-    from xmlrpclib import ServerProxy
+    from xmlrpclib import ServerProxy  # nosec
 except ImportError:
     from xmlrpc.client import ServerProxy
 
@@ -451,8 +451,8 @@ class OpenNebulaCloudConnector(CloudConnector):
                     sg_id = self._get_security_group(sg_name, auth_data)
                     if not sg_id:
                         sg_template = ""
-                        # open always SSH port on public nets
-                        if network.isPublic():
+                        # open always SSH port on public nets or with proxy_host
+                        if network.isPublic() or network.getValue("proxy_host"):
                             sg_template += "RULE = [ PROTOCOL = TCP, RULE_TYPE = inbound, RANGE = 22:22 ]\n"
 
                         outports = network.getOutPorts()
@@ -679,8 +679,8 @@ class OpenNebulaCloudConnector(CloudConnector):
         user_template = ""
         tags = self.get_instance_tags(system, auth_data, vm.inf)
         for key, value in tags.items():
-            key = key.replace("-", "_")
-            user_template += '%s = "%s", ' % (key, value)
+            key = key.replace("-", "_").replace(".", "_")
+            user_template += '"%s" = "%s", ' % (key, value)
 
         if user_template:
             res += "\nUSER_TEMPLATE = [%s]\n" % user_template[:-2]
