@@ -19,6 +19,8 @@ import time
 from netaddr import IPNetwork, IPAddress
 import os.path
 import tempfile
+import requests
+import base64
 
 try:
     from libcloud.common.exceptions import BaseHTTPError
@@ -120,7 +122,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                           "volume_url": None,
                           "api_version": "2.0",
                           "domain": None,
-                          "tenant_domain_id": None}
+                          "tenant_domain_id": None,
+                          "microversion": None}
 
             if 'username' in auth and 'password' in auth and 'tenant' in auth:
                 username = auth['username']
@@ -160,7 +163,7 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             for key, value in parameters.items():
                 if value:
                     if key in ['base_url', 'auth_token', 'service_type', 'image_url', 'volume_url',
-                               'network_url', 'service_region', 'auth_version', 'auth_url']:
+                               'network_url', 'service_region', 'auth_version', 'auth_url', 'microversion']:
                         key = 'ex_force_%s' % key
                     elif key == 'domain':
                         key = 'ex_domain_name'
@@ -193,6 +196,22 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             self.driver = driver
             return driver
 
+    @staticmethod
+    def guess_instance_type_sgx(size, epc_size_feature):
+        """Try to guess if this NodeSize has SGX support"""
+        epc_size = None
+        if epc_size_feature:
+            epc_size = epc_size_feature.getValue("m")
+        for k, v in size.extra.items():
+            if epc_size:
+                # If epc size is set only check it
+                if k.lower().find("sgx") != -1 and k.lower().find("epc_size") != 1 and int(v) >= epc_size:
+                    return True
+            else:
+                if k.lower().find("sgx") != -1 and v.lower() not in ['false', 'no', '0']:
+                    return True
+        return False
+
     @staticmethod
     def guess_instance_type_gpu(size, num_gpus, vendor=None, model=None):
         """Try to guess if this NodeSize has GPU support"""
@@ -227,12 +246,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         res = []
         for size in driver.list_sizes():
             # If extra specs are not fetch in the list_sizes function
-            if len(size.extra) == 0:
-                try:
-                    # get it now
-                    size.extra = driver.ex_get_size_extra_specs(size.id)
-                except Exception:
-                    self.log_exception("Error trying to get flavor '%s' extra_specs." % size.id)
+            try:
+                size.extra.update(driver.ex_get_size_extra_specs(size.id))
+            except Exception:
+                self.log_exception("Error trying to get flavor '%s' extra_specs." % size.id)
 
             size.extra['pci_devices'] = 0
             # Count to pci devices to enable sorting
@@ -246,7 +263,9 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                     else:
                         size.extra['pci_devices'] += 1
 
-            res.append(size)
+            # do not add disabled images
+            if 'disabled' not in size.extra or not size.extra['disabled']:
+                res.append(size)
 
         return res
 
@@ -258,6 +277,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         num_gpus = radl.getValue('gpu.count')
         gpu_model = radl.getValue('gpu.model')
         gpu_vendor = radl.getValue('gpu.vendor')
+        sgx = radl.getValue('cpu.sgx')
+        epc_size = radl.getFeature('cpu.sgx.epc_size')
 
         # get the node size with the lowest price, vcpus, memory and disk
         sizes.sort(key=lambda x: (x.price, x.vcpus, x.ram, x.disk, x.extra['pci_devices']))
@@ -267,6 +288,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             comparison = comparison and disk_free_op(size.disk, disk_free)
             if num_gpus and not self.guess_instance_type_gpu(size, num_gpus, gpu_vendor, gpu_model):
                 continue
+            if sgx == "yes" and not self.guess_instance_type_sgx(size, epc_size):
+                continue
 
             if comparison:
                 if not instance_type_name or size.name == instance_type_name:
@@ -419,6 +442,33 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
         except Exception as ex:
             self.log_warn("Error getting volume info: %s" % get_ex_error(ex))
 
+    def addAdditionalIP(self, vm, driver):
+        system = vm.info.systems[0]
+
+        # TODO: Manage if there are more that 1 port
+        if system.getValue('net_interface.0.additional_ip'):
+            additional_ip = system.getValue('net_interface.0.additional_ip')
+            # Find the port attached to the VM
+            port_bound = None
+            try:
+                for port in driver.ex_list_ports():
+                    if ("device_id" in port.extra and port.extra["device_id"] and
+                            port.extra["device_id"] == vm.id):
+                        port_bound = port
+            except Exception:
+                self.log_exception("Error getting port list")
+
+            if port_bound:
+                # set the additional IP
+                try:
+                    driver.ex_update_port(port_bound, allowed_address_pairs=[{"ip_address": additional_ip}])
+                    self.log_debug("Additional IP %s added to port %s" % (additional_ip, port_bound.id))
+                    system.delValue('net_interface.0.additional_ip')
+                except Exception:
+                    self.log_exception("Error adding addtional IP")
+            else:
+                self.log_warn("No port found for VM %s" % vm.id)
+
     def updateVMInfo(self, vm, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
@@ -443,8 +493,10 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
             except Exception as ex:
                 self.log_warn("Error updating VM info from flavor ID: %s" % get_ex_error(ex))
 
+            self.addAdditionalIP(vm, node.driver)
             self.addRouterInstance(vm, node.driver)
             self.setIPsFromInstance(vm, node)
+            self.add_dns_entries(vm, auth_data)
             self.setVolumesInfo(vm, node)
         else:
             self.log_warn("Error updating the instance %s. VM not found." % vm.id)
@@ -452,6 +504,38 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
 
         return (True, vm)
 
+    def add_dns_entries(self, vm, auth_data):
+        """
+        Add the required entries in the AWS Route53 service
+
+        Arguments:
+           - vm(:py:class:`IM.VirtualMachine`): VM information.
+           - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
+        """
+        try:
+            dns_entries = self.get_dns_entries(vm)
+            if dns_entries:
+                for hostname, domain, ip in dns_entries:
+                    # Special case for EGI DyDNS
+                    # format of the hostname: dydns:secret@hostname
+                    if hostname.startswith("dydns:") and "@" in hostname:
+                        parts = hostname[6:].split("@")
+                        auth = "%s.%s:%s" % (parts[1], domain[:-1], parts[0])
+                        headers = {"Authorization": "Basic %s" % base64.b64encode(auth.encode()).decode()}
+                        url = "https://nsupdate.fedcloud.eu/nic/update?hostname=%s.%s&myip=%s" % (parts[1],
+                                                                                                  domain[:-1],
+                                                                                                  ip)
+                        resp = requests.get(url, headers=headers)
+                        resp.raise_for_status()
+                    else:
+                        # TODO: https://docs.openstack.org/designate/latest/index.html
+                        return False
+            return True
+        except Exception as ex:
+            self.error_messages += "Error creating DNS entries %s.\n" % str(ex)
+            self.log_exception("Error creating DNS entries")
+            return False
+
     @staticmethod
     def map_radl_ost_networks(vm, ost_nets):
         """
@@ -1412,8 +1496,8 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
                 res.append(sg)
 
             try:
-                # open always SSH port on public nets
-                if network.isPublic():
+                # open always SSH port on public nets or private with proxy host
+                if network.isPublic() or network.getValue("proxy_host"):
                     driver.ex_create_security_group_rule(sg, 'tcp', 22, 22, '0.0.0.0/0')
                 # open all the ports for the VMs in the security group
                 driver.ex_create_security_group_rule(sg, 'tcp', 1, 65535, source_security_group=sg)
@@ -1660,14 +1744,11 @@ class OpenStackCloudConnector(LibCloudCloudConnector):
     def resizeVM(self, vm, radl, auth_data):
         node = self.get_node_with_id(vm.id, auth_data)
         if node:
-            new_cpu = radl.systems[0].getValue('cpu.count')
-            new_memory = radl.systems[0].getValue('memory.size')
-            instance_type = radl.systems[0].getValue('instance_type')
-            if not any([new_cpu, new_memory, instance_type]):
-                self.log_debug("No memory nor cpu nor instance_type specified. VM not resized.")
+            new_system = self.resize_vm_radl(vm, radl)
+            if not new_system:
                 return (True, "")
             else:
-                instance_type = self.get_instance_type(node.driver, radl.systems[0])
+                instance_type = self.get_instance_type(node.driver, new_system)
                 if instance_type is None:
                     return (False, "Error resizing VM: No instance type found.")
                 if node.extra['flavorId'] != instance_type.id:
