@@ -18,6 +18,7 @@ except ImportError:
 from toscaparser.tosca_template import ToscaTemplate
 from toscaparser.elements.interfaces import InterfacesDef
 from toscaparser.functions import Function, is_function, get_function, GetAttribute, Concat, Token
+from toscaparser.elements.scalarunit import ScalarUnit_Size
 from IM.ansible_utils import merge_recipes
 from radl.radl import (system, deploy, network, Feature, Features, configure,
                        contextualize_item, RADL, contextualize, ansible)
@@ -31,15 +32,18 @@ class Tosca:
 
     """
 
-    ARTIFACTS_PATH = os.path.dirname(
-        os.path.realpath(__file__)) + "/tosca-types/artifacts"
+    ARTIFACTS_PATH = os.path.dirname(os.path.realpath(__file__)) + "/tosca-types/artifacts"
     ARTIFACTS_REMOTE_REPO = "https://raw.githubusercontent.com/indigo-dc/tosca-types/master/artifacts/"
 
     logger = logging.getLogger('InfrastructureManager')
 
-    def __init__(self, yaml_str):
+    def __init__(self, yaml_str, verify=True):
         Tosca.logger.debug("TOSCA: %s" % yaml_str)
         self.yaml = yaml.safe_load(yaml_str)
+        if not verify:
+            def verify_fake(tpl):
+                return True
+            ToscaTemplate.verify_template = verify_fake
         self.tosca = ToscaTemplate(yaml_dict_tpl=copy.deepcopy(self.yaml))
 
     def serialize(self):
@@ -47,7 +51,8 @@ class Tosca:
 
     @staticmethod
     def deserialize(str_data):
-        return Tosca(str_data)
+        # avoid validation in this case
+        return Tosca(str_data, False)
 
     def _get_placement_property(self, sys_name, prop):
         """
@@ -364,6 +369,7 @@ class Tosca:
         # This is the solution using endpoints
         net_provider_id = None
         dns_name = None
+        additional_ip = None
         ports = {}
         endpoints = self._get_node_endpoints(node, nodetemplates)
 
@@ -396,16 +402,23 @@ class Tosca:
             if cap_props and "private_ip" in cap_props:
                 private_ip = self._final_function_result(cap_props["private_ip"].value, node)
             if cap_props and "ports" in cap_props:
-                ports = self._final_function_result(cap_props["ports"].value, node)
+                node_ports = self._final_function_result(cap_props["ports"].value, node)
+                if node_ports:
+                    for p in node_ports.values():
+                        ports[id(p)] = p
             if cap_props and "port" in cap_props:
                 port = self._final_function_result(cap_props["port"].value, node)
                 protocol = "tcp"
                 if "protocol" in cap_props:
                     protocol = self._final_function_result(cap_props["protocol"].value, node)
                 ports["im-%s-%s" % (protocol, port)] = {"protocol": protocol, "source": port}
+            if cap_props and "additional_ip" in cap_props:
+                additional_ip = self._final_function_result(cap_props["additional_ip"].value, node)
 
         if dns_name:
             system.setValue('net_interface.0.dns_name', dns_name)
+        if additional_ip:
+            system.setValue('net_interface.0.additional_ip', additional_ip)
 
         # Find associated Networks
         nets = self._get_bind_networks(node, nodetemplates)
@@ -413,18 +426,20 @@ class Tosca:
             # If there are network nodes, use it to define system network
             # properties
             port_net = None
-            for net_name, ip, dns_name, num in nets:
+            for net_name, ip, dns_name, num, additional_ip in nets:
                 net = radl.get_network_by_id(net_name)
                 if not net:
                     raise Exception("Node %s with a port binded to a non existing network: %s." % (node.name,
                                                                                                    net_name))
 
                 system.setValue('net_interface.%d.connection' % num, net_name)
-                # This is not a normative property
-                if dns_name:
-                    system.setValue('net_interface.%d.dns_name' % num, dns_name)
                 if ip:
                     system.setValue('net_interface.%d.ip' % num, ip)
+                # These are not normative properties
+                if dns_name:
+                    system.setValue('net_interface.%d.dns_name' % num, dns_name)
+                if additional_ip:
+                    system.setValue('net_interface.%d.additional_ip' % num, additional_ip)
 
                 if net.isPublic():
                     port_net = net
@@ -535,12 +550,13 @@ class Tosca:
         rel_tpls = src.get_relationship_template()
         rel_tpls.extend(trgt.get_relationship_template())
         for rel_tpl in rel_tpls:
-            if rel.type == rel_tpl.type:
-                return rel_tpl
-            else:
-                root_type = Tosca._get_root_parent_type(rel_tpl).type
-                if root_type == rel.type:
+            if rel_tpl.source.name == src.name and rel_tpl.target.name == trgt.name:
+                if rel.type == rel_tpl.type:
                     return rel_tpl
+                else:
+                    root_type = Tosca._get_root_parent_type(rel_tpl).type
+                    if root_type == rel.type:
+                        return rel_tpl
         return None
 
     @staticmethod
@@ -550,12 +566,13 @@ class Tosca:
             rel_tpl = Tosca._get_relationship_template(rel, src, trgt)
 
             rel_tlp_def_interfaces = {}
-            if rel_tpl.type_definition.interfaces and 'Standard' in rel_tpl.type_definition.interfaces:
-                rel_tlp_def_interfaces = rel_tpl.type_definition.interfaces['Standard']
+            for inteface_name in ['Standard', 'Configure']:
+                if rel_tpl.type_definition.interfaces and inteface_name in rel_tpl.type_definition.interfaces:
+                    rel_tlp_def_interfaces = rel_tpl.type_definition.interfaces[inteface_name]
 
             if src.name == node.name:
                 # Also add the configure of the target node of the relation
-                trgt_interfaces = Tosca._get_interfaces(trgt, ['pre_configure_source', 'post_configure_source'])
+                trgt_interfaces = Tosca._get_interfaces(trgt, steps=['pre_configure_source', 'post_configure_source'])
                 for name in ['pre_configure_source', 'post_configure_source', 'add_source']:
                     if trgt_interfaces and name in trgt_interfaces:
                         res[name] = trgt_interfaces[name]
@@ -569,7 +586,7 @@ class Tosca:
                                                   node_template=rel_tpl)
 
             elif trgt.name == node.name:
-                src_interfaces = Tosca._get_interfaces(src, ['pre_configure_target', 'post_configure_target'])
+                src_interfaces = Tosca._get_interfaces(src, steps=['pre_configure_target', 'post_configure_target'])
                 for name in ['pre_configure_target', 'post_configure_target', 'add_target',
                              'target_changed', 'remove_target']:
                     if src_interfaces and name in src_interfaces:
@@ -1240,46 +1257,12 @@ class Tosca:
         else:
             return 1
 
-    @staticmethod
-    def _unit_to_bytes(unit):
-        """Return the value of an unit."""
-        if not unit:
-            return 1
-        unit = unit.upper()
-
-        if unit.startswith("KI"):
-            return 1024
-        elif unit.startswith("K"):
-            return 1000
-        elif unit.startswith("MI"):
-            return 1048576
-        elif unit.startswith("M"):
-            return 1000000
-        elif unit.startswith("GI"):
-            return 1073741824
-        elif unit.startswith("G"):
-            return 1000000000
-        elif unit.startswith("TI"):
-            return 1099511627776
-        elif unit.startswith("T"):
-            return 1000000000000
-        else:
-            return 1
-
     @staticmethod
     def _get_size_and_unit(str_value):
         """
         Normalize the size and units to bytes
         """
-        parts = str_value.split(" ")
-        value = float(parts[0])
-        unit = 'M'
-        if len(parts) > 1:
-            unit = parts[1]
-
-        value = int(value * Tosca._unit_to_bytes(unit))
-
-        return value, 'B'
+        return ScalarUnit_Size(str_value).get_num_from_scalar_unit('B'), 'B'
 
     def _gen_network(self, node):
         """
@@ -1430,6 +1413,8 @@ class Tosca:
             'num_gpus': 'gpu.count',
             'gpu_vendor': 'gpu.vendor',
             'gpu_model': 'gpu.model',
+            'sgx': 'cpu.sgx',
+            'sgx_epc_size': 'cpu.sgx.epc_size'
         }
 
         for cap_type in ['os', 'host']:
@@ -1464,7 +1449,7 @@ class Tosca:
                                 raise Exception("User must be specified in the image credentials.")
                             name = "disk.0.os.credentials.username"
                             value = value['user']
-                        elif prop.name == "preemtible_instance":
+                        elif prop.name in ["preemtible_instance", "sgx"]:
                             value = 'yes' if value else 'no'
 
                         if isinstance(value, float) or isinstance(value, int):
@@ -1526,7 +1511,8 @@ class Tosca:
                     ip = self._final_function_result(port.get_property_value('ip_address'), port)
                     order = self._final_function_result(port.get_property_value('order'), port)
                     dns_name = self._final_function_result(port.get_property_value('dns_name'), port)
-                    nets.append((link, ip, dns_name, order))
+                    additional_ip = self._final_function_result(port.get_property_value('additional_ip'), port)
+                    nets.append((link, ip, dns_name, order, additional_ip))
 
         return nets
 
@@ -1611,7 +1597,8 @@ class Tosca:
                 return node_type
 
     @staticmethod
-    def _get_interfaces(node, steps=['create', 'configure', 'start', 'stop', 'delete']):
+    def _get_interfaces(node, interface_names=['Standard', 'Configure'],
+                        steps=['create', 'configure', 'start', 'stop', 'delete']):
         """
         Get a dict of InterfacesDef of the specified node
         """
@@ -1622,12 +1609,13 @@ class Tosca:
         node_type = node.type_definition
 
         while True:
-            if node_type.interfaces and 'Standard' in node_type.interfaces:
-                for name, elems in node_type.interfaces['Standard'].items():
-                    if name in steps:
-                        if name not in interfaces:
-                            interfaces[name] = InterfacesDef(node_type, 'Standard', name=name,
-                                                             value=elems, node_template=node)
+            for interface_name in interface_names:
+                if node_type.interfaces and interface_name in node_type.interfaces:
+                    for name, elems in node_type.interfaces[interface_name].items():
+                        if name in steps:
+                            if name not in interfaces:
+                                interfaces[name] = InterfacesDef(node_type, interface_name, name=name,
+                                                                 value=elems, node_template=node)
 
             if node_type.parent_type is not None:
                 node_type = node_type.parent_type
